@@ -24,6 +24,7 @@
 #include <limits.h>
 #include <math.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #include "conversions.h"
 
@@ -500,17 +501,285 @@ dc1394_bayer_HQLinear(const uint8_t *restrict bayer, uint8_t *restrict rgb, int
 dc1394error_t
 dc1394_bayer_EdgeSense(const uint8_t *restrict bayer, uint8_t *restrict rgb, int sx, int sy, int tile)
 {
-    /* Removed due to patent concerns */
-    return DC1394_FUNCTION_NOT_SUPPORTED;
+	uint8_t *outR, *outG, *outB;
+    register int i3, j3, base;
+    int i, j;
+    int dh, dv;
+    int tmp;
+	int sx3=sx*3;
+
+    // sx and sy should be even
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:
+    case DC1394_COLOR_FILTER_BGGR:
+        outR = &rgb[0];
+        outG = &rgb[1];
+        outB = &rgb[2];
+        break;
+    case DC1394_COLOR_FILTER_GBRG:
+    case DC1394_COLOR_FILTER_RGGB:
+        outR = &rgb[2];
+        outG = &rgb[1];
+        outB = &rgb[0];
+        break;
+    default:
+		return DC1394_INVALID_COLOR_FILTER;
+    }
+
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_GBRG:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outG[base]           = bayer[i + j];
+				outG[base + sx3 + 3] = bayer[i + j + sx + 1];
+				outR[base + 3]       = bayer[i + j + 1];
+				outB[base + sx3]     = bayer[i + j + sx];
+			}
+		}
+		// process GREEN channel
+		for (i3= 3*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		
+		for (i3=2*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >>1 ) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >>1 ) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		// process RED channel
+		for (i3=0; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outR[base]);
+			}
+		}
+
+		// process BLUE channel
+		for (i3=sx3; i3 < sy*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		break;
+
+    case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_RGGB:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outB[base] = bayer[i + j];
+				outR[base + sx3 + 3] = bayer[i + sx + (j + 1)];
+				outG[base + 3] = bayer[i + j + 1];
+				outG[base + sx3] = bayer[i + sx + j];
+			}
+		}
+		// process GREEN channel
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		for (i3=3*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >> 1) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >> 1) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP(tmp, outG[base]);
+			}
+		}
+		// process RED channel
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {        // G-points (1/2)
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >>1);
+				CLIP(tmp, outR[base]);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {        // G-points (2/2)
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outR[base]);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {        // B-points
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outR[base]);
+			}
+		}
+
+		// process BLUE channel
+		for (i = 0,i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 1, j3=3; j < sx - 2; j += 2, j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP(tmp, outB[base]);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP(tmp, outB[base]);
+			}
+		}
+		break;
+    }
+
+    ClearBorders(rgb, sx, sy, 3);
+
+    return DC1394_SUCCESS;
 }
 
 /* coriander's Bayer decoding */
 dc1394error_t
 dc1394_bayer_Downsample(const uint8_t *restrict bayer, uint8_t *restrict rgb, int sx, int sy, int tile)
 {
-    uint8_t *outR, *outG, *outB;
-    register int i, j;
-    int tmp;
+	uint8_t *outR, *outG, *outB;
+	register int i, j;
+	int tmp;
+	int st=sx*sy;
+	int p;
+	int sx2=sx<<1;
 
     switch (tile) {
     case DC1394_COLOR_FILTER_GRBG:
@@ -526,33 +795,35 @@ dc1394_bayer_Downsample(const uint8_t *restrict bayer, uint8_t *restrict rgb, in
         outB = &rgb[0];
         break;
     default:
-      return DC1394_INVALID_COLOR_FILTER;
+		return DC1394_INVALID_COLOR_FILTER;
     }
 
     switch (tile) {
     case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
     case DC1394_COLOR_FILTER_GBRG:
-        for (i = 0; i < sy*sx; i += (sx<<1)) {
+        for (i = 0; i < st; i += sx2) {
             for (j = 0; j < sx; j += 2) {
-                tmp = ((bayer[i + j] + bayer[i + sx + j + 1]) >> 1);
-                CLIP(tmp, outG[((i >> 2) + (j >> 1)) * 3]);
-                tmp = bayer[i + j + 1];
-                CLIP(tmp, outR[((i >> 2) + (j >> 1)) * 3]);
-                tmp = bayer[i + sx + j];
-                CLIP(tmp, outB[((i >> 2) + (j >> 1)) * 3]);
+				p=((i >> 2) + (j >> 1)) * 3;
+                tmp = bayer[i + j];
+				tmp+= bayer[i + sx + j + 1];
+				tmp>>=1;
+                outG[p] = (uint8_t)tmp;
+                outR[p] = bayer[i + j + 1];
+                outB[p] = bayer[i + sx + j];
             }
         }
         break;
     case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
     case DC1394_COLOR_FILTER_RGGB:
-        for (i = 0; i < sy*sx; i += (sx<<1)) {
+        for (i = 0; i < st; i += sx2) {
             for (j = 0; j < sx; j += 2) {
-                tmp = ((bayer[i + sx + j] + bayer[i + j + 1]) >> 1);
-                CLIP(tmp, outG[((i >> 2) + (j >> 1)) * 3]);
-                tmp = bayer[i + sx + j + 1];
-                CLIP(tmp, outR[((i >> 2) + (j >> 1)) * 3]);
-                tmp = bayer[i + j];
-                CLIP(tmp, outB[((i >> 2) + (j >> 1)) * 3]);
+				p=((i >> 2) + (j >> 1)) * 3;
+                tmp = bayer[i + sx + j];
+				tmp+= bayer[i + j + 1];
+				tmp>>= 1;
+                outG[p] = (uint8_t)tmp;
+                outR[p] = bayer[i + sx + j + 1];
+                outB[p] = bayer[i + j];
             }
         }
         break;
@@ -1048,8 +1319,273 @@ dc1394_bayer_HQLinear_uint16(const uint16_t *restrict bayer, uint16_t *restrict
 dc1394error_t
 dc1394_bayer_EdgeSense_uint16(const uint16_t *restrict bayer, uint16_t *restrict rgb, int sx, int sy, int tile, int bits)
 {
-    /* Removed due to patent concerns */
-    return DC1394_FUNCTION_NOT_SUPPORTED;
+	uint16_t *outR, *outG, *outB;
+    register int i3, j3, base;
+    int i, j;
+    int dh, dv;
+    int tmp;
+	int sx3=sx*3;
+
+    // sx and sy should be even
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:
+    case DC1394_COLOR_FILTER_BGGR:
+        outR = &rgb[0];
+        outG = &rgb[1];
+        outB = &rgb[2];
+        break;
+    case DC1394_COLOR_FILTER_GBRG:
+    case DC1394_COLOR_FILTER_RGGB:
+        outR = &rgb[2];
+        outG = &rgb[1];
+        outB = &rgb[0];
+        break;
+    default:
+		return DC1394_INVALID_COLOR_FILTER;
+    }
+
+    switch (tile) {
+    case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_GBRG:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outG[base]           = bayer[i + j];
+				outG[base + sx3 + 3] = bayer[i + j + sx + 1];
+				outR[base + 3]       = bayer[i + j + 1];
+				outB[base + sx3]     = bayer[i + j + sx];
+			}
+		}
+		// process GREEN channel
+		for (i3= 3*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		
+		for (i3=2*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >>1 ) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >>1 ) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		// process RED channel
+		for (i3=0; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+
+		// process BLUE channel
+		for (i3=sx3; i3 < sy*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		break;
+
+    case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
+    case DC1394_COLOR_FILTER_RGGB:
+        // copy original RGB data to output images
+		for (i = 0, i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 0, j3=0; j < sx; j += 2, j3+=6) {
+				base=i3+j3;
+				outB[base] = bayer[i + j];
+				outR[base + sx3 + 3] = bayer[i + sx + (j + 1)];
+				outG[base + 3] = bayer[i + j + 1];
+				outG[base + sx3] = bayer[i + sx + j];
+			}
+		}
+		// process GREEN channel
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=6; j3 < sx3 - 9; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outB[base - 6] +
+						   outB[base + 6]) >> 1) -
+						   outB[base]);
+				dv = abs(((outB[base - (sx3<<1)] +
+						   outB[base + (sx3<<1)]) >> 1) -
+						   outB[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		for (i3=3*sx3; i3 < (sy - 3)*sx3; i3 += (sx3<<1)) {
+			for (j3=9; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				dh = abs(((outR[base - 6] +
+						   outR[base + 6]) >> 1) -
+						   outR[base]);
+				dv = abs(((outR[base - (sx3<<1)] +
+						   outR[base + (sx3<<1)]) >> 1) -
+						   outR[base]);
+				tmp = (((outG[base - 3]   + outG[base + 3]) >> 1) * (dh<=dv) +
+					   ((outG[base - sx3] + outG[base + sx3]) >> 1) * (dh>dv));
+				//tmp = (dh==dv) ? tmp>>1 : tmp;
+				CLIP16(tmp, outG[base], bits);
+			}
+		}
+		// process RED channel
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {        // G-points (1/2)
+			for (j3=6; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - 3] -
+					  outG[base - 3] +
+					  outR[base + 3] -
+					  outG[base + 3]) >>1);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+		for (i3=2*sx3; i3 < (sy - 2)*sx3; i3 += (sx3<<1)) {
+			for (j3=3; j3 < sx3; j3+=6) {        // G-points (2/2)
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3] -
+					  outG[base - sx3] +
+					  outR[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outR[base], bits);
+			}
+			for (j3=6; j3 < sx3 - 3; j3+=6) {        // B-points
+				base=i3+j3;
+				tmp = outG[base] +
+					((outR[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outR[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outR[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outR[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outR[base], bits);
+			}
+		}
+
+		// process BLUE channel
+		for (i = 0,i3=0; i < sy*sx; i += (sx<<1), i3 += (sx3<<1)) {
+			for (j = 1, j3=3; j < sx - 2; j += 2, j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - 3] -
+					  outG[base - 3] +
+					  outB[base + 3] -
+					  outG[base + 3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		for (i3=sx3; i3 < (sy - 1)*sx3; i3 += (sx3<<1)) {
+			for (j3=0; j3 < sx3 - 3; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3] -
+					  outG[base - sx3] +
+					  outB[base + sx3] -
+					  outG[base + sx3]) >> 1);
+				CLIP16(tmp, outB[base], bits);
+			}
+			for (j3=3; j3 < sx3 - 6; j3+=6) {
+				base=i3+j3;
+				tmp = outG[base] +
+					((outB[base - sx3 - 3] -
+					  outG[base - sx3 - 3] +
+					  outB[base - sx3 + 3] -
+					  outG[base - sx3 + 3] +
+					  outB[base + sx3 - 3] -
+					  outG[base + sx3 - 3] +
+					  outB[base + sx3 + 3] -
+					  outG[base + sx3 + 3]) >> 2);
+				CLIP16(tmp, outB[base], bits);
+			}
+		}
+		break;
+    }
+
+    ClearBorders_uint16(rgb, sx, sy, 3);
+
+    return DC1394_SUCCESS;
 }
 
 /* coriander's Bayer decoding */
@@ -1059,6 +1595,9 @@ dc1394_bayer_Downsample_uint16(const uint16_t *restrict bayer, uint16_t *restric
     uint16_t *outR, *outG, *outB;
     register int i, j;
     int tmp;
+	int st=sx*sy;
+	int sx2=sx<<1;
+	int p;
 
     switch (tile) {
     case DC1394_COLOR_FILTER_GRBG:
@@ -1080,36 +1619,35 @@ dc1394_bayer_Downsample_uint16(const uint16_t *restrict bayer, uint16_t *restric
     switch (tile) {
     case DC1394_COLOR_FILTER_GRBG:        //---------------------------------------------------------
     case DC1394_COLOR_FILTER_GBRG:
-        for (i = 0; i < sy*sx; i += (sx<<1)) {
+        for (i = 0; i < st; i += sx2) {
             for (j = 0; j < sx; j += 2) {
-                tmp =
-                    ((bayer[i + j] + bayer[i + sx + j + 1]) >> 1);
-                CLIP16(tmp, outG[((i >> 2) + (j >> 1)) * 3], bits);
-                tmp = bayer[i + sx + j + 1];
-                CLIP16(tmp, outR[((i >> 2) + (j >> 1)) * 3], bits);
-                tmp = bayer[i + sx + j];
-                CLIP16(tmp, outB[((i >> 2) + (j >> 1)) * 3], bits);
+				p=((i >> 2) + (j >> 1)) * 3;
+                tmp = bayer[i + j];
+				tmp+= bayer[i + sx + j + 1];
+				tmp>>=1;
+                outG[p] = (uint16_t)tmp;
+                outR[p] = bayer[i + j + 1];
+                outB[p] = bayer[i + sx + j];
             }
         }
         break;
     case DC1394_COLOR_FILTER_BGGR:        //---------------------------------------------------------
     case DC1394_COLOR_FILTER_RGGB:
-        for (i = 0; i < sy*sx; i += (sx<<1)) {
+        for (i = 0; i < st; i += sx2) {
             for (j = 0; j < sx; j += 2) {
-                tmp =
-                    ((bayer[i + sx + j] + bayer[i + j + 1]) >> 1);
-                CLIP16(tmp, outG[((i >> 2) + (j >> 1)) * 3], bits);
-                tmp = bayer[i + sx + j + 1];
-                CLIP16(tmp, outR[((i >> 2) + (j >> 1)) * 3], bits);
-                tmp = bayer[i + j];
-                CLIP16(tmp, outB[((i >> 2) + (j >> 1)) * 3], bits);
+				p=((i >> 2) + (j >> 1)) * 3;
+                tmp = bayer[i + sx + j];
+				tmp+= bayer[i + j + 1];
+				tmp>>= 1;
+                outG[p] = (uint16_t)tmp;
+                outR[p] = bayer[i + sx + j + 1];
+                outB[p] = bayer[i + j];
             }
         }
         break;
     }
 
     return DC1394_SUCCESS;
-
 }
 
 /* coriander's Bayer decoding */
