@@ -41,7 +41,7 @@
 #include "config.h"
 #include "internal.h"
 #include "macosx/macosx.h"
-#include "macosx/capture.h"
+
 
 /**********************/
 /* Internal functions */
@@ -206,7 +206,7 @@ callback (buffer_info * buffer, NuDCLRef dcl)
 }
 
 static void
-socket_callback (CFSocketRef s, CFSocketCallBackType type,
+socket_callback_firewire (CFSocketRef s, CFSocketCallBackType type,
                  CFDataRef address, const void * data, void * info)
 {
     platform_camera_t * craw = info;
@@ -306,9 +306,9 @@ servicing_thread (void * cam_ptr)
     IOFireWireLibDeviceRef d = craw->iface;
 
     (*d)->AddCallbackDispatcherToRunLoopForMode (d, CFRunLoopGetCurrent (),
-                                                 kCFRunLoopDefaultMode);
+                                                 kCFRunLoopCommonModes);
     (*d)->AddIsochCallbackDispatcherToRunLoopForMode (d, CFRunLoopGetCurrent (),
-                                                      kCFRunLoopDefaultMode);
+                                                      kCFRunLoopCommonModes);
 
     MPSignalSemaphore(craw->capture.thread_init_semaphore);
 
@@ -321,12 +321,13 @@ servicing_thread (void * cam_ptr)
  CAPTURE SETUP
 **************************************************************/
 dc1394error_t
-platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
+dc1394_macosx_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
                      uint32_t flags)
 {
     dc1394capture_t * capture = &(craw->capture);
     dc1394camera_t * camera = craw->camera;
     dc1394error_t err;
+    IOReturn ret;
     IOFireWireLibDeviceRef d = craw->iface;
     IOFWSpeed speed;
     IOFireWireLibIsochChannelRef chan;
@@ -375,15 +376,15 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
     pipe (capture->notify_pipe);
 
     capture->socket = CFSocketCreateWithNative (NULL, capture->notify_pipe[0],
-                                                kCFSocketReadCallBack, socket_callback, &socket_context);
+                                                kCFSocketReadCallBack, socket_callback_firewire, &socket_context);
     /* Set flags so that the underlying fd is not closed with the socket */
     CFSocketSetSocketFlags (capture->socket,
                             CFSocketGetSocketFlags (capture->socket) & ~kCFSocketCloseOnInvalidate);
     capture->socket_source = CFSocketCreateRunLoopSource (NULL,
                                                           capture->socket, 0);
     if (!capture->run_loop)
-        dc1394_capture_schedule_with_runloop (camera,
-                                              CFRunLoopGetCurrent (), kCFRunLoopDefaultMode);
+        dc1394_macosx_capture_schedule_with_runloop (craw,
+                                              CFRunLoopGetCurrent (), kCFRunLoopCommonModes);
     CFRunLoopAddSource (capture->run_loop, capture->socket_source,
                         capture->run_loop_mode);
 
@@ -406,7 +407,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
                                      capture->frames[0].packet_size, speed,
                                      CFUUIDGetUUIDBytes (kIOFireWireIsochChannelInterfaceID));
     if (!chan) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not create IsochChannelInterface");
         return DC1394_FAILURE;
     }
@@ -415,7 +416,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
     rem_port = (*d)->CreateRemoteIsochPort (d, true,
                                             CFUUIDGetUUIDBytes (kIOFireWireRemoteIsochPortInterfaceID));
     if (!rem_port) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not create RemoteIsochPortInterface");
         return DC1394_FAILURE;
     }
@@ -435,7 +436,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
     dcl_pool = (*d)->CreateNuDCLPool (d, numdcls,
                                       CFUUIDGetUUIDBytes (kIOFireWireNuDCLPoolInterfaceID));
     if (!dcl_pool) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not create NuDCLPoolInterface");
         return DC1394_FAILURE;
     }
@@ -443,7 +444,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
 
     dcl_program = CreateDCLProgram (craw);
     if (!dcl_program) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not create DCL Program");
         return DC1394_FAILURE;
     }
@@ -452,7 +453,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
                                            kFWDCLSyBitsEvent, 1, 1, nil, 0, &(capture->databuf), 1,
                                            CFUUIDGetUUIDBytes (kIOFireWireLocalIsochPortInterfaceID));
     if (!loc_port) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not create LocalIsochPortInterface");
         return DC1394_FAILURE;
     }
@@ -465,15 +466,19 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
     (*chan)->AddListener (chan, (IOFireWireLibIsochPortRef) loc_port);
     (*chan)->SetTalker (chan, (IOFireWireLibIsochPortRef) rem_port);
 
-    if ((*chan)->AllocateChannel (chan) != kIOReturnSuccess) {
-        platform_capture_stop (craw);
-        dc1394_log_error("Could not allocate channel");
+    if ((ret = (*chan)->AllocateChannel (chan)) != kIOReturnSuccess) {
+        dc1394_macosx_capture_stop (craw);
+        if (ret == kIOReturnNoSpace)
+            dc1394_log_error("Not enough iso bandwidth or channels are "
+                    "available to begin capture");
+        else
+            dc1394_log_error("Could not allocate channel");
         return DC1394_FAILURE;
     }
     capture->iso_is_allocated = 1;
 
     if ((*chan)->Start (chan) != kIOReturnSuccess) {
-        platform_capture_stop (craw);
+        dc1394_macosx_capture_stop (craw);
         dc1394_log_error("Could not start channel");
         return DC1394_FAILURE;
     }
@@ -497,7 +502,7 @@ platform_capture_setup(platform_camera_t *craw, uint32_t num_dma_buffers,
 *****************************************************/
 
 dc1394error_t
-platform_capture_stop(platform_camera_t *craw)
+dc1394_macosx_capture_stop(platform_camera_t *craw)
 {
     dc1394camera_t * camera = craw->camera;
     dc1394capture_t * capture = &(craw->capture);
@@ -599,7 +604,7 @@ platform_capture_stop(platform_camera_t *craw)
 //#define PREV_BUFFER(c,i) (((i) == 0) ? (c)->num_frames-1 : ((i)-1))
 
 dc1394error_t
-platform_capture_dequeue (platform_camera_t * craw,
+dc1394_macosx_capture_dequeue (platform_camera_t * craw,
                         dc1394capture_policy_t policy,
                         dc1394video_frame_t **frame)
 {
@@ -609,6 +614,11 @@ platform_capture_dequeue (platform_camera_t * craw,
     dc1394video_frame_t * frame_tmp = capture->frames + next;
     char ch;
 
+	if(craw->capture.frames==NULL || craw->capture_is_set==0) {
+		*frame=NULL;
+		return DC1394_CAPTURE_IS_NOT_SET;
+	}
+
     if ( (policy<DC1394_CAPTURE_POLICY_MIN) || (policy>DC1394_CAPTURE_POLICY_MAX) )
         return DC1394_INVALID_CAPTURE_POLICY;
 
@@ -648,7 +658,7 @@ platform_capture_dequeue (platform_camera_t * craw,
 
 
 dc1394error_t
-platform_capture_enqueue (platform_camera_t * craw,
+dc1394_macosx_capture_enqueue (platform_camera_t * craw,
                         dc1394video_frame_t * frame)
 {
     dc1394capture_t * capture = &(craw->capture);
@@ -692,7 +702,7 @@ platform_capture_enqueue (platform_camera_t * craw,
 }
 
 dc1394bool_t
-platform_capture_is_frame_corrupt (platform_camera_t * craw,
+dc1394_macosx_capture_is_frame_corrupt (platform_camera_t * craw,
         dc1394video_frame_t * frame)
 {
     dc1394capture_t * capture = &(craw->capture);
@@ -705,7 +715,7 @@ platform_capture_is_frame_corrupt (platform_camera_t * craw,
 }
 
 int
-platform_capture_get_fileno (platform_camera_t * craw)
+dc1394_macosx_capture_get_fileno (platform_camera_t * craw)
 {
     dc1394capture_t * capture = &(craw->capture);
 
@@ -715,33 +725,28 @@ platform_capture_get_fileno (platform_camera_t * craw)
     return capture->notify_pipe[0];
 }
 
-int
-dc1394_capture_schedule_with_runloop (dc1394camera_t * camera,
+dc1394error_t
+dc1394_macosx_capture_schedule_with_runloop (platform_camera_t * craw,
         CFRunLoopRef run_loop, CFStringRef run_loop_mode)
 {
-    dc1394camera_priv_t * cpriv = DC1394_CAMERA_PRIV (camera);
-    platform_camera_t * craw = cpriv->pcam;
     dc1394capture_t * capture = &(craw->capture);
-
     if (craw->capture_is_set) {
         dc1394_log_warning("schedule_with_runloop must be called before capture_setup");
-        return -1;
+        return DC1394_FAILURE;
     }
 
     capture->run_loop = run_loop;
     capture->run_loop_mode = run_loop_mode;
-    return 0;
+    return DC1394_SUCCESS;
 }
 
-void
-dc1394_capture_set_callback (dc1394camera_t * camera,
+dc1394error_t
+dc1394_macosx_capture_set_callback (platform_camera_t * craw,
                              dc1394capture_callback_t callback, void * user_data)
 {
-    dc1394camera_priv_t * cpriv = DC1394_CAMERA_PRIV (camera);
-    platform_camera_t * craw = cpriv->pcam;
     dc1394capture_t * capture = &(craw->capture);
-
     capture->callback = callback;
     capture->callback_user_data = user_data;
+    return DC1394_SUCCESS;
 }
 
